try:
    from functions.game_selection import get_game, get_diff, get_mode
    from functions.setup_results import setup, end_screen
    from functions.refresh_update import refresh_screen, update_status, find_start_word
except:
    from typetype.functions.game_selection import get_game, get_diff, get_mode
    from typetype.functions.setup_results import setup, end_screen
    from typetype.functions.refresh_update import refresh_screen, update_status, find_start_word
import curses
import time
import random
import os
import argparse



def main(stdscr):
    
    #get command line args
    cursor = False
    parser = argparse.ArgumentParser(description='Typing Game')
    parser.add_argument('-cursor', action='store_true', help='Enable cursor mode')
    args = parser.parse_args()
    if args.cursor:
        #display | cursor
        cursor = True
    
    #start and setup colors and use the default terminal color as the background
    curses.start_color()
    curses.use_default_colors()
    stdscr.clear()
    #grey
    curses.init_pair(1, 235, -1)
    #white
    curses.init_pair(2, curses.COLOR_WHITE, -1)
    #(error)
    curses.init_pair(3, 160, -1)
    #(extra)
    #change color red to a greyish salmon color
    curses.init_color(curses.COLOR_RED, 300, 150, 150)
    curses.init_pair(4, curses.COLOR_RED, -1)
    #magenta
    curses.init_pair(5, curses.COLOR_MAGENTA, -1)
    
    #prompt user for what type of game they want
    choices = get_game(stdscr)
    if choices == 'q':
        return 
    choices = get_diff(stdscr, choices)
    if 'q' in choices:
        return
    if 'x' not in choices:
        choices = get_mode(stdscr, choices)
    if 'q' in choices:
        return
    response = start_game(stdscr, choices, cursor)
    while True:
        if response == 'r':
            response = start_game(stdscr, choices, cursor)
        elif response == 'm':
            choices = get_game(stdscr)
            if choices == 'q':
                return
            choices = get_diff(stdscr, choices)
            if 'q' in choices:
                return
            if 'x' not in choices:
                choices = get_mode(stdscr, choices)
            if 'q' in choices:
                return
            response = start_game(stdscr, choices, cursor)
        elif response == 'q':
            break
    return









#this is where each indivudual game is played
def start_game(stdscr, choices, cursor):
    
    #get the sentence based on the users choices
    sentence = setup(stdscr, choices)
    #set some variables
    playing_game = False
    time_started = False
    start_time = 0
    end_time = 0
    onword = 0
    errors = 0
    stdscr.timeout(-1)
    
    #determine if it's a timed game:
    timed = 0
    if choices[0] == 't':
        if choices[2] == '1':
            timed = 15
        if choices[2] == '2':
            timed = 30
        if choices[2] == '3':
            timed = 60
    
    #list of valid characters that can be used while typing
    allchars = "qwertyuiopasdfghjklzxcvbnm QWERTYUIOPASDFGHJKLZXCVBNM1234567890!@#$%^&*()-=_+[]{}|;:\'\",./<>?`~\\"
    valid_characters = [thing for thing in allchars]
    
    #create a dictionary for the status of each character in each word
    status = {}
    
    #a list of every word in the sentence NO SPACES
    original_words = sentence.split(" ")
    
    #initialize staus dictionary
    for count, words in enumerate(original_words):
        status[count] = ['u' for _ in range(len(words))]

    #make a list of empty strings for each word in the sentence. The list will be populated as the user types
    new_words = ["" for words in original_words]
    
    #make a list of empty strings for the overflowing characters for each word
    extras = ["" for words in original_words]
    
    
    
    
    #initialize the screen with the message and the initial sentence
    refresh_screen(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor)
    
    #loop that waits for user input
    while True:
        
        #allow the getch() function to timeout so we can refresh the screen every 0.1 seconds
        stdscr.timeout(100)
        
        #wait for an input key
        key = stdscr.getch()
        try:
            char = chr(key)
        except:
            char = 'NA'
        
        #end the game if the timer has passed the time limit
        if timed!=0 and time_started:
            if time.time()-start_time>timed:
                break
        
        #if the function times out, refresh the screen
        if key == -1:
            refresh_screen(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor)
            continue
            
        
        #if the key is not a valid character, the delete key, or the esc key: go to the start of the loop
        #127 is the delete key and 27 is the esc key
        if char not in valid_characters and key!=127 and key!=27:
            continue
        
        
        #start time when the user starts typing for the first time
        if not time_started:
            #* will return to menu if the game hasn't started
            if key == 42:
                return 'm'
            time_started = True
            playing_game = True
            start_time = time.time()
        
        
        #if the delete key is pressed
        if key == 127:
            #deleting from the current word
            if len(extras[onword])>0:
                extras[onword] = extras[onword][:-1]
            elif len(new_words[onword])>0:
                new_words[onword] = new_words[onword][:-1]
            #move to previous word if the cursor is at the beginning of a word (if its not the first word)
            elif onword>0:
                onword -= 1
            
                
        #esc key restarts the game with the same settings
        elif key == 27:
            return start_game(stdscr, choices, cursor)
        
        
        
        #add character to extras list
        elif len(new_words[onword])>=len(original_words[onword]) and char!= ' ':
            if len(extras[onword])<10:
                extras[onword] = extras[onword] + char
                errors+=1
        #add character to the word that the user is typing
        elif char!=' ':
            new_words[onword] = new_words[onword] + char
            #if the character is wrong: increment the errors variable
            if original_words[onword][len(new_words[onword])-1] != char:
                errors+=1
            #if the last word is typed completely correctly, end the game
            if onword == len(original_words)-1 and new_words[onword] == original_words[onword]:
                status = update_status(status, original_words, new_words, onword)
                break
        #space means that the word is finished and we are on to the next word
        if char == ' ':
            #if letters are skipped then add to the error count.
            if len(original_words[onword])>len(new_words[onword]):
                errors+=1
            onword+=1
            #if we pass the last word then end the game
            if onword >= len(original_words):
                break
        
        
        #update the status dictionary
        status = update_status(status, original_words, new_words, onword)
        #refresh the screen
        refresh_screen(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor)
    
    #the time it took the user to finish
    end_time = time.time()-start_time
    #show the user the results on the end screen.
    user_response = end_screen(stdscr, original_words, new_words, extras, end_time, errors, status, timed, choices)
    #return the user response to main
    return user_response
    

#used to call main
def callmain():
    #this is to take away the escape delay
    os.environ.setdefault('ESCDELAY', '25')
    #run main
    curses.wrapper(main)





import curses
import time
import random
import os


#clears the screen and updates it with the most recent data
def refresh_screen(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor):
    #hide the cursor and clear the screen
    if cursor:
        curses.curs_set(0)
    else:
        curses.curs_set(1)
    stdscr.clear()
    cursorplaced = False
    #get the first word that we have to start at (we only show 2 or 3 lines at a time)
    firstword = find_start_word(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor)
    #this is how many lines we're going to display
    linesleft = 3 if firstword!=-1 else 2
    
    #adjust variables according to the screen dimensions
    height, width = stdscr.getmaxyx()
    line_start = int(width/6)
    line_width = int(width/6*4)
    start_height = int(height/4)
    #if the game hasn't started, then display a message
    if not playing_game:
        try:
            stdscr.addstr(start_height-2, int((width-len("start typing..."))/2), "start typing...")
        except: 
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
    
    #add character by character to the display
    current_x = line_start
    current_y = start_height
    cursor_x = line_start
    cursor_y = start_height
    wordcount = firstword if firstword !=-1 else 0
    
    #loop for each word in the text
    while wordcount<len(original_words):
        
        lettercount = 0
        #if the cursor is on this word, then take into account the width of the cursor
        if cursor and onword == wordcount:
            cursorwidth = 1
        else:
            cursorwidth = 0
        #if the word wont fit on this line, then move it down to the next
        if (current_x+cursorwidth+len(original_words[wordcount])+len(extras[wordcount]))>(line_start+line_width):
            current_y += 1
            linesleft -= 1
            if linesleft<1:
                break
            current_x = line_start
        #for each letter in the word
        while lettercount<len(original_words[wordcount]):
            #set the color of the letter based on the status of the letter
            if status[wordcount][lettercount]=='u':
                color = 1
            elif status[wordcount][lettercount]=='r':
                color = 2
            if status[wordcount][lettercount]=='w':
                color = 3
            if status[wordcount][lettercount]=='s':
                color = 1
            #place cursor if this is the right place
            if cursorplaced==False and (wordcount == onword) and (len(extras[wordcount])==0) and lettercount>=len(new_words[wordcount]):
                if not cursor:
                    cursor_y = current_y
                    cursor_x = current_x
                    cursorplaced = True
                else:
                    try:
                        stdscr.addstr(current_y, current_x, "|", curses.color_pair(2))
                    except:
                        stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
                    cursorplaced = True
                    current_x+=1
            #place letter on the screen
            try:
                stdscr.addstr(current_y, current_x, original_words[wordcount][lettercount], curses.color_pair(color))
            except:
                stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
            current_x+=1
            lettercount+=1
        #if there are letters in extra then we can place it now
        for count, letter in enumerate(extras[wordcount]):
            try:
                stdscr.addstr(current_y, current_x, extras[wordcount][count], curses.color_pair(4))
            except:
                stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
            current_x+=1
        #place cursor if this is the right place
        if (onword==wordcount) and cursorplaced == False:
            if not cursor:
                cursor_x = current_x
                cursor_y = current_y
                cursorplaced = True
            else:
                try:
                    stdscr.addstr(current_y, current_x, "|", curses.color_pair(2))
                except:
                    stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
                cursorplaced = True
                current_x+=1
        #add space between each word
        try:
            stdscr.addstr(current_y, current_x, " ", curses.color_pair(3))
        except:
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
        current_x+=1
        wordcount+=1
    
    #add the time to the screen
    time_diff = int(time.time() - start_time) if playing_game else 0
    if timed>0:
        time_diff = int(timed-time_diff) if (timed-time_diff)>0 else 0
    try:
        stdscr.addstr(current_y+2, int((width-len(str(time_diff)))/2), str(time_diff), curses.color_pair(2))
    except:
        stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
    #put to the screen
    stdscr.move(cursor_y,cursor_x)
    stdscr.refresh()
    



#-----------------------------------------------------------------------------------------------------





#updates the contents of the status dictionary
def update_status(status, original_words, new_words, onword):
    status2 = status
    i = onword-1 if onword>0 else 0
    while i<=onword:
        word = new_words[i]
        correct = original_words[i]
        #for each letter in the full word
        for count, correctchar in enumerate(correct):
            #if the user hasn't typed this part of the word, then break
            if count >= len(word):
                break
            #if the letter is right
            if correctchar == word[count]:
                status2[i][count] = 'r'
            #if the letter is wrong
            elif correctchar != word[count]:
                status2[i][count] = 'w'
        lastcount = len(word)
        if i!=onword:
            #if the letter was skipped then it will be here
            while lastcount<len(correct):
                status2[i][lastcount] = 's'
                lastcount+=1
        else:
            #this is for the word that you are on, for the letters after the cursor
            while lastcount<len(correct):
                status2[i][lastcount] = 'u'
                lastcount+=1
        i+=1
    #return the new and updated version of status
    return status2










#-------------------------------------------------------------------------------------------

#this function will find the word that we have to start displaying from 
#I want to display one line above and below where the cursor is
def find_start_word(stdscr, status, original_words, playing_game, new_words, extras, onword, start_time, timed, cursor):
    #hide the cursor and clear the screen
    cursorplaced = False
    
    #adjust variables according to the screen dimensions
    height, width = stdscr.getmaxyx()
    line_start = int(width/6)
    line_width = int(width/6*4)
    start_height = int(height/4)
    
    startword = 0
    previndex = 0
    
    #add character by character to the display
    current_x = line_start
    current_y = start_height
    wordcount = 0
    
    #loop for each word in the text
    while wordcount<len(original_words):
        lettercount = 0
        if cursor and onword == wordcount:
            cursorwidth = 1
        else:
            cursorwidth = 0
        #if the word wont fit on this line, then move it down to the next
        if (current_x+cursorwidth+len(original_words[wordcount])+len(extras[wordcount]))>(line_start+line_width):
            current_y += 1
            previndex = startword
            startword = wordcount
            current_x = line_start
        #for each letter in the word
        while lettercount<len(original_words[wordcount]):
            #place cursor if this is the right place
            if cursorplaced==False and (wordcount == onword) and (len(extras[wordcount])==0) and lettercount>=len(new_words[wordcount]):
                cursorplaced = True
                if startword == 0:
                    return -1
                return previndex
                current_x+=1
            #place letter on the screen
            current_x+=1
            lettercount+=1
        #if there are letters in extra then we can place it now
        for count, letter in enumerate(extras[wordcount]):
            current_x+=1
        #place cursor if this is the right place
        if (onword==wordcount) and cursorplaced == False:
            cursorplaced = True
            if startword == 0:
                return -1
            return previndex
            current_x+=1
        #add space between each word
        current_x+=1
        wordcount+=1
    




import curses
import time
import random
import os


#setup the colors that will be used for the text and return the text for the user
def setup(stdscr, choices):

    #choose sentence from sentences
    if choices[0] == 'x':
        lines = open(os.path.dirname(os.path.dirname(__file__ ))+"/words/text.txt").readlines()
    elif choices[0] != 'x' and choices[1] == 'e':
        lines = open(os.path.dirname(os.path.dirname(__file__)) +"/words/200words.txt").readlines()
    elif choices[0] != 'x' and choices[1] == 'm':
        lines = open(os.path.dirname(os.path.dirname(__file__)) +"/words/5000words.txt").readlines()
    elif choices[0] != 'x' and choices[1] == 'h':
        lines = open(os.path.dirname(os.path.dirname(__file__)) +"/words/25000words.txt").readlines()
        
    #text
    if choices[0] == 'x' and choices[1] == 's':
        chosen = random.choice(lines)[:-1]
        while len(chosen)>160:
            chosen = random.choice(lines)[:-1]
    elif choices[0] == 'x' and choices[1] == 'm':
        chosen = random.choice(lines)[:-1]
        while len(chosen)>300 or len(chosen)<=160:
            chosen = random.choice(lines)[:-1]
    elif choices[0] == 'x' and choices[1] == 'l':
        chosen = random.choice(lines)[:-1]
        while len(chosen)<=300:
            chosen = random.choice(lines)[:-1]
    if choices[0] == 'x':
        return chosen
    
    #timed or words
    last10 = []
    sentence = ""
    needed = 200
    
    if choices[0] == 't' and choices[2] == '1':
        needed = 50
    elif choices[0] == 't' and choices[2] == '2':
        needed = 100
    elif choices[0] == 'w' and choices[2] == '1':
        needed = 10
    elif choices[0] == 'w' and choices[2] == '2':
        needed = 25
    elif choices[0] == 'w' and choices[2] == '3':
        needed = 50
    elif choices[0] == 'w' and choices[2] == '4':
        needed = 100
    
    #get the need ammount of words from the file
    while len(sentence.split(" "))<needed:
        randomword = random.choice(lines)[:-1]
        #don't let the same word repeat close to each other
        if randomword in last10:
            continue
        if len(last10) == 10:
            last10.pop(0)
        last10.append(randomword)
        if sentence!= "":
            randomword = " " + randomword
        sentence+=randomword
    
    return sentence






#---------------------------------------------------------------------------------------------------------------------






def end_screen(stdscr, original_words, new_words, extras, end_time, errorcount, status, timed, choices):
    
    #if its timed then set the end_time to the time
    if end_time>timed and timed:
        end_time = timed
    
    #count the number of r's in status
    correct_chars = 0
    i = 0
    while i<len(status):
        for stat in status[i]:
            if stat == 'r':
                correct_chars+=1
        i+=1
        

    #count the number of correct words typed
    correct_words = sum([1 if ( (word == new_words[count]) and len(extras[count])==0 ) else 0 for count, word in enumerate(original_words)])
    
    #take into account the spaces placed after correct words (dont count the last word)
    correct_chars += correct_words if correct_words != len(original_words) else (correct_words-1)
    
    
            
            
    #calculate results
    cpm = int(correct_chars*60.0/end_time)
    wpm = round(cpm/5.0,2) #round(correct_words*60.0/end_time ,2)
    accuracy = round((correct_chars)*1.0/(correct_chars+errorcount)*100,2)
    
    
    new_high = False
    scoreindex = 0
    #get the old highscore a list of incorrect words
    lines = open(os.path.dirname(os.path.dirname(__file__ ))+"/words/highscores.txt").readlines()
    if choices[0] == 't':
        if choices[1] == 'm':
            scoreindex += 3
        elif choices[1] == 'h':
            scoreindex += 6
    elif choices[0] == 'w':
        if choices[1] == 'e':
            scoreindex += 9
        elif choices[1] == 'm':
            scoreindex += 13
        elif choices[1] == 'h':
            scoreindex += 17
    try:
        if choices[2] == '2':
            scoreindex += 1
        elif choices[2] == '3':
            scoreindex += 2
        elif choices[2] == '4':
            scoreindex += 3
    except:
        scoreindex = -1
        
    if scoreindex != -1:
        old_high = float(lines[scoreindex][:-1])
        #update the highscore if it is beat
        if wpm > old_high:
            new_high = True
            f = open(os.path.dirname(os.path.dirname(__file__ ))+"/words/highscores.txt", "w")
            for count, line in enumerate(lines):
                if count == scoreindex:
                    f.write(str(wpm)+"\n")
                else:
                    f.write(line)
            f.close()
    

    
    
    
    
    active = 1
    
    while True:
        height, width = stdscr.getmaxyx()
        curses.curs_set(0)
        startheight = int(height/2.0)
        stdscr.clear()
        resultheight = int(height/4.0)
        resultx = int((width-len("wpm: {}      ".format(wpm))-len("acc: {}%".format(accuracy)))/2)+1
        try:
            if new_high:
                stdscr.addstr(resultheight-2, int((width-len("NEW HIGH SCORE"))/2), "NEW HIGH SCORE", curses.color_pair(5))
                stdscr.addstr(resultheight, resultx, "wpm: {}      ".format(wpm), curses.color_pair(5))
                stdscr.addstr(resultheight, resultx+len("wpm: {}      ".format(wpm)), "acc: {}%".format(accuracy), curses.color_pair(2))
                stdscr.addstr(resultheight+2, resultx, "time: {}s      ".format(round(end_time,1)), curses.color_pair(2))
                stdscr.addstr(resultheight+2, resultx+len("wpm: {}      ".format(wpm)), "cpm: {}".format(cpm), curses.color_pair(2))
            else: 
                stdscr.addstr(resultheight, resultx, "wpm: {}      ".format(wpm), curses.color_pair(2))
                stdscr.addstr(resultheight, resultx+len("wpm: {}      ".format(wpm)), "acc: {}%".format(accuracy), curses.color_pair(2))
                stdscr.addstr(resultheight+2, resultx, "time: {}s      ".format(round(end_time,1)), curses.color_pair(2))
                stdscr.addstr(resultheight+2, resultx+len("wpm: {}      ".format(wpm)), "cpm: {}".format(cpm), curses.color_pair(2))
            
            if active == 1:
                stdscr.addstr(startheight, int((width-len("restart game"))/2), "restart game", curses.color_pair(2))
            else:
                stdscr.addstr(startheight, int((width-len("restart game"))/2), "restart game", curses.color_pair(1))
            if active == 2:
                stdscr.addstr(startheight+2, int((width-len("back to menu"))/2), "back to menu", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+2, int((width-len("back to menu"))/2), "back to menu", curses.color_pair(1))
            if active == 3:
                stdscr.addstr(startheight+4, int((width-len("quit"))/2), "quit", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+4, int((width-len("quit"))/2), "quit", curses.color_pair(1))
            
                
        except:
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
        stdscr.refresh()
            
        #wait for an input key
        stdscr.timeout(-1)
        key = stdscr.getch()
        
        #down,up,left,right,space,enter: 258,259,260,261,32,10
        #next
        if key == 258 or key == 261:
            if active<3:
                active+=1
        #previous
        elif key == 259 or key == 260:
            if active>1:
                active-=1
        #select
        elif key == 10:
            if active == 1:
                return 'r'
            elif active == 2:
                return 'm'
            elif active == 3:
                return 'q'
        #esc
        elif key == 27:
            return 'q'
        





from setuptools import setup, find_packages
import sys

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='typetype',
    version='1.0.7',
    author='Ahmet Ozer',
    url="https://github.com/ahmet8zer/typetype",
    description='A command line typing game',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'typetype=typetype.ahmetsgame:callmain'
        ]
    },
    install_requires=[
] + (['windows-curses'] if sys.platform == 'win32' else [])
)


import curses
import time
import random
import os




#user selects which game mode they want to play
def get_game(stdscr):
    active = 1
    while True:
        height, width = stdscr.getmaxyx()
        curses.curs_set(0)
        stdscr.clear()
        startheight = int(height/3)
        startheight = 4 if startheight<4 else startheight
        try:
            stdscr.addstr(1, int((width-4)/2), "type", curses.color_pair(2))
            stdscr.addstr(2, int((width-4)/2), "ǝdʎʇ", curses.color_pair(2))
            if active == 1:
                stdscr.addstr(startheight, int((width-len("timed mode"))/2), "timed mode", curses.color_pair(2))
            else:
                stdscr.addstr(startheight, int((width-len("timed mode"))/2), "timed mode", curses.color_pair(1))
            if active == 2:
                stdscr.addstr(startheight+2, int((width-len("words mode"))/2), "words mode", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+2, int((width-len("words mode"))/2), "words mode", curses.color_pair(1))
            if active == 3:
                stdscr.addstr(startheight+4, int((width-len("quote mode"))/2), "quote mode", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+4, int((width-len("quote mode"))/2), "quote mode", curses.color_pair(1))
            if active == 4:
                stdscr.addstr(startheight+7, int((width-len("quit"))/2), "quit", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+7, int((width-len("quit"))/2), "quit", curses.color_pair(1))
            
        except:
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
        
        stdscr.refresh()
            
        #wait for an input key
        stdscr.timeout(-1)

        key = stdscr.getch()
        #down,up,left,right,space,enter: 258,259,260,261,32,10
        #next
        if key == 258 or key == 261:
            if active<4:
                active+=1
        #previous
        elif key == 259 or key == 260:
            if active>1:
                active-=1
        #enter
        elif key == 10:
            if active == 1:
                return 't'
            elif active == 2:
                return 'w'
            elif active == 3:
                return 'x'
            elif active == 4:
                return 'q'
        #esc
        elif key == 27:
            return 'q'











#user selects which difficulty they want
def get_diff(stdscr, choice):
    if 'q' in choice:
        return choice
    active = 1
    while True:
        height, width = stdscr.getmaxyx()
        sml = int((width-4)/2)
        startheight = int(height/3)
        startheight = 4 if startheight<4 else startheight
        curses.curs_set(0)
        stdscr.clear()
        try:
            stdscr.addstr(1, int((width-4)/2), "type", curses.color_pair(2))
            stdscr.addstr(2, int((width-4)/2), "ǝdʎʇ", curses.color_pair(2))
            if choice == 't' or choice == 'w':
                if active == 1:
                    stdscr.addstr(startheight, int((width-len("easy"))/2), "easy", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight, int((width-len("easy"))/2), "easy", curses.color_pair(1))
                if active == 2:
                    stdscr.addstr(startheight+2, int((width-len("medium"))/2), "medium", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+2, int((width-len("medium"))/2), "medium", curses.color_pair(1))
                if active ==3:
                    stdscr.addstr(startheight+4, int((width-len("hard"))/2), "hard", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+4, int((width-len("hard"))/2), "hard", curses.color_pair(1))
            elif choice == 'x':
                if active == 1:
                    stdscr.addstr(startheight, int((width-4)/2), "short", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight, int((width-4)/2), "short", curses.color_pair(1))
                if active == 2:
                    stdscr.addstr(startheight+2, int((width-4)/2), "medium", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+2, int((width-4)/2), "medium", curses.color_pair(1))
                if active == 3:
                    stdscr.addstr(startheight+4, int((width-len("long"))/2), "long", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+4, int((width-len("long"))/2), "long", curses.color_pair(1))
            if active == 4:
                stdscr.addstr(startheight+7, int((width-len("back"))/2), "back", curses.color_pair(2))
            else:
                stdscr.addstr(startheight+7, int((width-len("back"))/2), "back", curses.color_pair(1))
        except:
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
        
        stdscr.refresh()
            
        #wait for an input key
        stdscr.timeout(-1)
        key = stdscr.getch()
        #down,up,left,right,space,enter: 258,259,260,261,32,10
        
        #next
        if key == 258 or key == 261:
            if active<4:
                active+=1
        #previous
        elif key == 259 or key == 260:
            if active>1:
                active-=1
        #enter
        elif key == 10:
            if active == 1 and (choice == 't' or choice == 'w'):
                return choice + 'e'
            elif active == 2:
                return choice + 'm'
            elif active == 3 and (choice == 't' or choice == 'w'):
                return choice + 'h'
            elif active == 4:
                return get_diff(stdscr, get_game(stdscr))
            elif active == 1 and (choice == 'x'):
                return choice + 's'
            elif active == 3 and (choice == 'x'):
                return choice + 'l'
        #delete
        elif key == 127:
            return get_diff(stdscr, get_game(stdscr))
        








#get the time or word number that the user wants
def get_mode(stdscr, choice):
    if 'q' in choice:
        return choice
    active = 1
    active_max = 4 if choice[0]=='t' else 5
    while True:
        height, width = stdscr.getmaxyx()
        startheight = int(height/3)
        startheight = 4 if startheight<4 else startheight
        
        curses.curs_set(0)
        stdscr.clear()
        try:
            stdscr.addstr(1, int((width-4)/2), "type", curses.color_pair(2))
            stdscr.addstr(2, int((width-4)/2), "ǝdʎʇ", curses.color_pair(2))
            if choice[0] == 't':
                if active == 1:
                    stdscr.addstr(startheight, int((width-len("15s "))/2), "15s", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight, int((width-len("15s "))/2), "15s", curses.color_pair(1))
                if active == 2:
                    stdscr.addstr(startheight+2, int((width-len("30s "))/2), "30s", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+2, int((width-len("30s "))/2), "30s", curses.color_pair(1))
                if active == 3:
                    stdscr.addstr(startheight+4, int((width-len("60s "))/2), "60s", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+4, int((width-len("60s "))/2), "60s", curses.color_pair(1))
                if active == 4:
                    stdscr.addstr(startheight+7, int((width-len("back"))/2), "back", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+7, int((width-len("back"))/2), "back", curses.color_pair(1))
            elif choice[0] == 'w':
                if active == 1:
                    stdscr.addstr(startheight, int((width-len("10 words"))/2), "10 words", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight, int((width-len("10 words"))/2), "10 words", curses.color_pair(1))
                if active == 2:
                    stdscr.addstr(startheight+2, int((width-len("25 words"))/2), "25 words", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+2, int((width-len("25 words"))/2), "25 words", curses.color_pair(1))
                if active == 3:
                    stdscr.addstr(startheight+4, int((width-len("50 words"))/2), "50 words", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+4, int((width-len("50 words"))/2), "50 words", curses.color_pair(1))
                if active == 4:
                    stdscr.addstr(startheight+6, int((width-len("100 word"))/2), "100 words", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+6, int((width-len("100 word"))/2), "100 words", curses.color_pair(1))
                if active == 5:
                    stdscr.addstr(startheight+9, int((width-len("back"))/2), "back", curses.color_pair(2))
                else:
                    stdscr.addstr(startheight+9, int((width-len("back"))/2), "back", curses.color_pair(1))
        except:
            stdscr.addstr(0, 0, "Window is too small!", curses.color_pair(4))
        
        stdscr.refresh()
        
        #wait for an input key
        stdscr.timeout(-1)
        key = stdscr.getch()
        #down,up,left,right,space,enter: 258,259,260,261,32,10
        
        #next
        if key == 258 or key == 261:
            if active<active_max:
                active+=1
        #previous
        elif key == 259 or key == 260:
            if active>1:
                active-=1
        #select
        elif key == 10:
            if active == 1:
                return choice + '1'
            elif active == 2:
                return choice + '2'
            elif active == 3:
                return choice + '3'
            elif active == 4 and choice[0] == 't':
                return get_mode(stdscr, get_diff(stdscr, choice[:-1]))
            elif active == 14 and choice[0] == 'w':
                return choice + '4'
            elif active == 5:
                return get_mode(stdscr, get_diff(stdscr, choice[:-1]))
        #delete
        elif key == 127:
            return get_mode(stdscr, get_diff(stdscr, choice[:-1]))
        
        